
/*---------------------------------------------------------------------------
      Chapter 8: Intelligent techniques of Logic Programming (Paragraph 8.2)
      This code is part of the book
      LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
      Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
      Hellenic Academic On-line Publications, 2015
    ----------------------------------------------------------------------------*/
    
/** Non-Deterministic algorithms   **/

/* Two lists with a common element  */

common_element(List1,List2):-
  list_member(X,List1),
  list_member(X,List2).

/* sorting a listing by finding the right permutation */
list_sort(L,SL):- 
	permutate(L,SL), 
	isordered(SL).

permutate([],[]).
permutate(L,[H|PL]):- 
	list_delete(H,L,NL), 
	permutate(NL,PL).

isordered([]).
isordered([X]).
isordered([X1,X2|T]):-
	X1 =< X2,
	isordered([X2|T]).

/* The 8 queens problem  */

solve(QL):- 
	template(QL), 
	solution(QL).

template([[1,Y1],[2,Y2],[3,Y3],[4,Y4],[5,Y5],[6,Y6],[7,Y7],[8,Y8]]).

solution([]).
solution([[X,Y]|Rest]):-
	member(Y, [a,b,c,d,e,f,g,h]),
	solution(Rest),
	safe([X,Y],Rest).

num(a,1).
num(b,2).
num(c,3).
num(d,4).
num(e,5).
num(f,6).
num(g,7).
num(h,8).

safe(_,[]).
safe([X,Y],[[Xa,Ya]|Rest]):-
	num(Y,Yn),
	num(Ya,Yan),
	Yn=\=Yan,
	abs(X-Xa)=\=abs(Yn-Yan), % could be: X-Xa =\=Yn-Yan, X-Xa =\=Yan-Yn,
	safe([X,Y],Rest).

number_of_solutions(N):-
	setof(QL,solve(QL),L),length(L,N).